(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     41668,       1245]
NotebookOptionsPosition[     39101,       1150]
NotebookOutlinePosition[     39457,       1166]
CellTagsIndexPosition[     39414,       1163]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"This", " ", "notebook", " ", "uses", " ", 
    RowBox[{"Mathematica", "'"}], "s", " ", "symbolic", " ", "mathematics", 
    " ", "tools", " ", "to", " ", "derive", " ", "or", " ", "verify", " ", 
    "a", " ", "variety", " ", "of", " ", "formulae", " ", "that", " ", 
    "appear", " ", "either", " ", "in", " ", "the", " ", "paper", " ", "or", 
    " ", "in", " ", "the", " ", "solution", " ", "code"}], " ", "*)"}], 
  "\n"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Analytical", " ", "formula", " ", "for", " ", "finite", " ", "value", " ",
     "of", " ", "autarky", " ", "measure", " ", 
    RowBox[{"(", "FVA", ")"}], " ", "that", " ", "applies", " ", "when", " ", 
    "distribution", " ", "is", " ", "lognormal"}], " ", "*)"}], "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"ClearAll", "[", "\"\<Global`*\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Integrate", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Rho]"}], ")"}]}], " ", 
      RowBox[{
       RowBox[{"PDF", "[", 
        RowBox[{"LogNormalDistribution", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{"\[Sigma]", "^", "2"}], ")"}]}], "/", "2"}], ",", 
          "\[Sigma]"}], "]"}], "]"}], "[", "x", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "Infinity"}], "}"}], ",", 
     RowBox[{"Assumptions", "->", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"Re", "[", 
          RowBox[{"\[Sigma]", "^", "2"}], "]"}], ">", "0"}], ",", 
        RowBox[{
         RowBox[{"Re", "[", "\[Sigma]", "]"}], ">", "0"}]}], "}"}]}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5164728548125887`*^9, 3.516472860950014*^9}}],

Cell[BoxData[
 SuperscriptBox["\[ExponentialE]", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "\[Rho]"}], ")"}], " ", "\[Rho]", " ", 
   SuperscriptBox["\[Sigma]", "2"]}]]], "Output",
 CellChangeTimes->{{3.504789208290902*^9, 3.504789219266588*^9}, 
   3.515057938474078*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Features", " ", "of", " ", "the", " ", "normalized", " ", "version", " ",
      "of", " ", "the", " ", "value", " ", "function", " ", "which", " ", 
     "Theorem", " ", "1", " ", "proves", " ", "is", " ", "\[Digamma]"}], "-", 
    "bounded"}], " ", "*)"}], "\n", "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"ClearAll", "[", "\"\<Global`*\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"vNorm", "  ", "=", " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"v", "[", "m", "]"}]}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Eta]", "+", 
        SuperscriptBox["m", 
         RowBox[{"1", "-", "\[Rho]"}]]}], ")"}]}], "]"}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Log", " ", "of", " ", "the", " ", 
     RowBox[{"(", 
      RowBox[{"negative", " ", "of", " ", "the"}], ")"}], " ", 
     "normalization", " ", "in", " ", "the", " ", "paper"}], " ", "*)"}], 
   "\[IndentingNewLine]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"Log", "[", 
  RowBox[{"-", 
   FractionBox[
    RowBox[{"v", "[", "m", "]"}], 
    RowBox[{
     SuperscriptBox["m", 
      RowBox[{"1", "-", "\[Rho]"}]], "+", "\[Eta]"}]]}], "]"}]], "Output",
 CellChangeTimes->{3.515057940361178*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vNormP", " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{"D", "[", 
    RowBox[{"vNorm", ",", "m"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Rho]"}], ")"}], " ", 
    RowBox[{"v", "[", "m", "]"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"m", "+", 
      RowBox[{
       SuperscriptBox["m", "\[Rho]"], " ", "\[Eta]"}]}], ")"}], " ", 
    RowBox[{
     SuperscriptBox["v", "\[Prime]",
      MultilineFunction->None], "[", "m", "]"}]}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{
      SuperscriptBox["m", "\[Rho]"], " ", "\[Eta]"}]}], ")"}], " ", 
   RowBox[{"v", "[", "m", "]"}]}]]], "Output",
 CellChangeTimes->{3.5150579449214582`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InputForm", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"vNormP", " ", "/.", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
       "vPPVecExcBot"}]}], " ", "/.", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
      "vPVecExcBot"}]}], " ", "/.", " ", 
    RowBox[{
     RowBox[{"v", "[", "m", "]"}], " ", "\[Rule]", " ", "vVecExcBot"}]}], " ",
    "/.", " ", 
   RowBox[{"m", " ", "\[Rule]", " ", "mVecExcBot"}]}], "]"}]], "Input"],

Cell["\<\
(vPVecExcBot*(mVecExcBot + mVecExcBot^\[Rho]*\[Eta]) + 
  vVecExcBot*(-1 + \[Rho]))/(vVecExcBot*
  (mVecExcBot + mVecExcBot^\[Rho]*\[Eta]))\
\>", "Output",
 CellChangeTimes->{3.5150579468173637`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vNormPP", " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{"D", "[", 
    RowBox[{"vNormP", ",", "m"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.501988003503849*^9, 3.501988129928754*^9}, {
  3.502159703949891*^9, 3.502159710444455*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Rho]"}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{"m", "+", 
      RowBox[{
       SuperscriptBox["m", "\[Rho]"], " ", "\[Eta]", " ", "\[Rho]"}]}], ")"}],
     " ", 
    SuperscriptBox[
     RowBox[{"v", "[", "m", "]"}], "2"]}], "-", 
   RowBox[{"m", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", 
       RowBox[{
        SuperscriptBox["m", "\[Rho]"], " ", "\[Eta]"}]}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["v", "\[Prime]",
       MultilineFunction->None], "[", "m", "]"}], "2"]}], "+", 
   RowBox[{"m", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", 
       RowBox[{
        SuperscriptBox["m", "\[Rho]"], " ", "\[Eta]"}]}], ")"}], "2"], " ", 
    RowBox[{"v", "[", "m", "]"}], " ", 
    RowBox[{
     SuperscriptBox["v", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "m", "]"}]}]}], 
  RowBox[{"m", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"m", "+", 
      RowBox[{
       SuperscriptBox["m", "\[Rho]"], " ", "\[Eta]"}]}], ")"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"v", "[", "m", "]"}], "2"]}]]], "Output",
 CellChangeTimes->{3.515057950130905*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Put", " ", "the", " ", "result", " ", "in", " ", "a", " ", "format", " ",
      "that", " ", "can", " ", "be", " ", "copied"}], "-", "and", "-", 
    RowBox[{"pasted", " ", "into", " ", "Mathematica", " ", "notebook"}]}], 
   " ", "*)"}], 
  RowBox[{"InputForm", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"vNormPP", "  ", "/.", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["v", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
         "vPPVecExcBot"}]}], " ", "/.", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
        "vPVecExcBot"}]}], " ", "/.", " ", 
      RowBox[{
       RowBox[{"v", "[", "m", "]"}], " ", "\[Rule]", " ", "vVecExcBot"}]}], 
     " ", "/.", " ", 
     RowBox[{"m", " ", "\[Rule]", " ", "mVecExcBot"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.515164593677114*^9, 3.515164621753561*^9}}],

Cell["\<\
(-(mVecExcBot*vPVecExcBot^2*
    (mVecExcBot + mVecExcBot^\[Rho]*\[Eta])^2) + 
  mVecExcBot*vPPVecExcBot*vVecExcBot*
   (mVecExcBot + mVecExcBot^\[Rho]*\[Eta])^2 - 
  vVecExcBot^2*(-1 + \[Rho])*(mVecExcBot + 
    mVecExcBot^\[Rho]*\[Eta]*\[Rho]))/
 (mVecExcBot*vVecExcBot^2*
  (mVecExcBot + mVecExcBot^\[Rho]*\[Eta])^2)\
\>", "Output",
 CellChangeTimes->{3.5150579515664454`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", "beginCDCPrivate", " ", "*)"}], "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"ClearAll", "[", "\"\<Global`*\>\"", "]"}], ";", 
    RowBox[{"vOu", " ", "=", " ", 
     RowBox[{
      RowBox[{"v", "[", "m", "]"}], "/", 
      RowBox[{"u", "[", "m", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"vOuP", " ", "=", " ", 
    RowBox[{"Simplify", "[", 
     RowBox[{"D", "[", 
      RowBox[{"vOu", ",", "m"}], "]"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"v", "[", "m", "]"}]}], " ", 
    RowBox[{
     SuperscriptBox["u", "\[Prime]",
      MultilineFunction->None], "[", "m", "]"}]}], "+", 
   RowBox[{
    RowBox[{"u", "[", "m", "]"}], " ", 
    RowBox[{
     SuperscriptBox["v", "\[Prime]",
      MultilineFunction->None], "[", "m", "]"}]}]}], 
  SuperscriptBox[
   RowBox[{"u", "[", "m", "]"}], "2"]]], "Output",
 CellChangeTimes->{{3.502263021892997*^9, 3.5022630586053963`*^9}, 
   3.502263248918792*^9, 3.503944143332881*^9, 3.51505795587222*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InputForm", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["v", "\[Prime]",
     MultilineFunction->None], "[", "m", "]"}], " ", "/.", " ", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"vOuPVal", " ", "\[Equal]", " ", "vOuP"}], ",", 
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "m", "]"}]}], "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.50226527292588*^9, 3.502265344757965*^9}}],

Cell["(vOuPVal*u[m]^2 + v[m]*Derivative[1][u][m])/u[m]", "Output",
 CellChangeTimes->{{3.502265308454348*^9, 3.502265345236401*^9}, 
   3.503944144609943*^9, 3.51505795723982*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InputForm", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"vOuP", "  ", "/.", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["u", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
         RowBox[{"uPP", "[", "m", "]"}]}]}], " ", "/.", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
        RowBox[{"uP", "[", "m", "]"}]}]}], " ", "/.", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
       "vPPVecExcBot"}]}], " ", "/.", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
      "vPVecExcBot"}]}], " ", "/.", " ", 
    RowBox[{
     RowBox[{"v", "[", "m", "]"}], " ", "\[Rule]", " ", "vVecExcBot"}]}], " ",
    "/.", " ", 
   RowBox[{"m", " ", "\[Rule]", " ", "mVecExcBot"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5022635856847277`*^9, 3.5022636029913073`*^9}}],

Cell["\<\
(vPVecExcBot*u[mVecExcBot] - 
  vVecExcBot*uP[mVecExcBot])/u[mVecExcBot]^2\
\>", "Output",
 CellChangeTimes->{{3.5022635920627537`*^9, 3.502263603511448*^9}, 
   3.503944145656584*^9, 3.515057958307699*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vOuPP", " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{"D", "[", 
    RowBox[{"vOuP", ",", "m"}], "]"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.502263064506501*^9, 3.502263155182817*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"v", "[", "m", "]"}], " ", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox["u", "\[Prime]",
       MultilineFunction->None], "[", "m", "]"}], "2"]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"u", "[", "m", "]"}], " ", 
    RowBox[{
     SuperscriptBox["u", "\[Prime]",
      MultilineFunction->None], "[", "m", "]"}], " ", 
    RowBox[{
     SuperscriptBox["v", "\[Prime]",
      MultilineFunction->None], "[", "m", "]"}]}], "-", 
   RowBox[{
    RowBox[{"u", "[", "m", "]"}], " ", 
    RowBox[{"v", "[", "m", "]"}], " ", 
    RowBox[{
     SuperscriptBox["u", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "m", "]"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"u", "[", "m", "]"}], "2"], " ", 
    RowBox[{
     SuperscriptBox["v", "\[Prime]\[Prime]",
      MultilineFunction->None], "[", "m", "]"}]}]}], 
  SuperscriptBox[
   RowBox[{"u", "[", "m", "]"}], "3"]]], "Output",
 CellChangeTimes->{{3.5022630744657583`*^9, 3.50226309793904*^9}, 
   3.502263156075683*^9, 3.502263256699546*^9, 3.503944146823*^9, 
   3.515057959392322*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InputForm", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"vOuPP", " ", "/.", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["u", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
         RowBox[{"uPP", "[", "m", "]"}]}]}], " ", "/.", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
        RowBox[{"uP", "[", "m", "]"}]}]}], " ", "/.", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
       "vPPVecExcBot"}]}], " ", "/.", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
      "vPVecExcBot"}]}], " ", "/.", " ", 
    RowBox[{
     RowBox[{"v", "[", "m", "]"}], " ", "\[Rule]", " ", "vVecExcBot"}]}], " ",
    "/.", " ", 
   RowBox[{"m", " ", "\[Rule]", " ", "mVecExcBot"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.502263161756343*^9, 3.5022631990798197`*^9}, {
  3.502263264052623*^9, 3.502263437650799*^9}, {3.5022659052288237`*^9, 
  3.502265908851757*^9}}],

Cell["\<\
(vPPVecExcBot*u[mVecExcBot]^2 - 
  2*vPVecExcBot*u[mVecExcBot]*uP[mVecExcBot] + 
  2*vVecExcBot*uP[mVecExcBot]^2 - 
  vVecExcBot*u[mVecExcBot]*uPP[mVecExcBot])/
 u[mVecExcBot]^3\
\>", "Output",
 CellChangeTimes->{
  3.502263200060985*^9, {3.502263290504977*^9, 3.5022633034663687`*^9}, {
   3.502263359213407*^9, 3.50226340333549*^9}, 3.502263438282551*^9, 
   3.5022659099523153`*^9, 3.5039441478669662`*^9, 3.515057960559512*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "\"\<Global`*\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"vGap", " ", "=", " ", 
  RowBox[{"Log", "[", 
   RowBox[{"1", "-", 
    RowBox[{
     RowBox[{"vLim", "[", 
      RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}], "/", 
     RowBox[{"v", "[", 
      RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}]}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.5023178923832283`*^9, 3.5023179161513453`*^9}, {
   3.502318033730933*^9, 3.502318034876453*^9}, {3.5023192195426073`*^9, 
   3.502319224260206*^9}, 3.502444981521474*^9, {3.5024493486466293`*^9, 
   3.502449387781969*^9}}],

Cell[BoxData[
 RowBox[{"Log", "[", 
  RowBox[{"1", "-", 
   FractionBox[
    RowBox[{"vLim", "[", 
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], 
    RowBox[{"v", "[", 
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]]}], "]"}]], "Output",
 CellChangeTimes->{{3.5024493791940002`*^9, 3.502449388264604*^9}, 
   3.503944148902933*^9, 3.504780108318904*^9, 3.515057961660236*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vFromGap", "=", 
  RowBox[{
   RowBox[{"vLim", "[", 
    RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     RowBox[{"Exp", "[", "vGap", "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.504780103734502*^9, 3.5047801679753237`*^9}}],

Cell[BoxData[
 RowBox[{"v", "[", 
  SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]], "Output",
 CellChangeTimes->{{3.504780127167595*^9, 3.50478016898589*^9}, 
   3.515057962711507*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vGapP", "=", 
  RowBox[{"D", "[", 
   RowBox[{"vGap", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.502317920660077*^9, 3.502317929797151*^9}, {
   3.502317978624797*^9, 3.502317984607325*^9}, 3.502318824027355*^9, {
   3.502444982668213*^9, 3.5024449837922297`*^9}, {3.5024493919031973`*^9, 
   3.502449393856447*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
     RowBox[{"vLim", "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], " ", 
     RowBox[{
      SuperscriptBox["v", "\[Prime]",
       MultilineFunction->None], "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], 
    SuperscriptBox[
     RowBox[{"v", "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "2"]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
     RowBox[{
      SuperscriptBox["vLim", "\[Prime]",
       MultilineFunction->None], "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], 
    RowBox[{"v", "[", 
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]]}], 
  RowBox[{"1", "-", 
   FractionBox[
    RowBox[{"vLim", "[", 
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], 
    RowBox[{"v", "[", 
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]]}]]], "Output",
 CellChangeTimes->{3.50231793056746*^9, 3.5023180368421373`*^9, 
  3.502318858429722*^9, 3.5023192272116413`*^9, 3.502319584101799*^9, 
  3.502444993841343*^9, 3.5024493944697*^9, 3.503944149893828*^9, 
  3.504780184933723*^9, 3.5150579637138357`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InputForm", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"vGapP", " ", "/.", " ", 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", "\[Rule]", 
            "m"}]}], " ", "/.", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["vLim", "\[Prime]",
             MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
           "vLimPVecExcBot"}]}], "/.", " ", 
         RowBox[{
          RowBox[{"vLim", "[", "m", "]"}], " ", "\[Rule]", " ", 
          "vLimVecExcBot"}]}], "/.", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["u", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
         RowBox[{"uPP", "[", "m", "]"}]}]}], " ", "/.", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
        RowBox[{"uP", "[", "m", "]"}]}]}], " ", "/.", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
       "vPPVecExcBot"}]}], " ", "/.", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
      "vPVecExcBot"}]}], " ", "/.", " ", 
    RowBox[{
     RowBox[{"v", "[", "m", "]"}], " ", "\[Rule]", " ", "vVecExcBot"}]}], " ",
    "/.", " ", 
   RowBox[{"m", " ", "\[Rule]", " ", "mVecExcBot"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5023179965236177`*^9, 3.502318004491074*^9}, {
   3.502318050996833*^9, 3.5023182706294737`*^9}, 3.502444984639988*^9, {
   3.502449409995488*^9, 3.502449427878263*^9}}],

Cell["\<\
((mVecExcBot*vLimVecExcBot*vPVecExcBot)/
   vVecExcBot^2 - (mVecExcBot*vLimPVecExcBot)/
   vVecExcBot)/(1 - vLimVecExcBot/vVecExcBot)\
\>", "Output",
 CellChangeTimes->{{3.502318019577043*^9, 3.50231806840248*^9}, {
   3.502318160760393*^9, 3.502318271376006*^9}, 3.502318860267741*^9, 
   3.502319230362279*^9, 3.502319681983029*^9, 3.5024449966433773`*^9, {
   3.502449400178499*^9, 3.502449428882022*^9}, 3.50394415100458*^9, 
   3.5047801904431458`*^9, 3.5150579649968967`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vGapPP", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{"D", "[", 
    RowBox[{"vGapP", ",", "\[Mu]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.502318831960434*^9, 3.502318868904224*^9}, {
  3.502318913050722*^9, 3.502318916626889*^9}, {3.502444985656365*^9, 
  3.50244498649329*^9}, {3.502449459541514*^9, 3.5024494630099287`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
       SuperscriptBox[
        RowBox[{"vLim", "[", 
         SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{
         SuperscriptBox["v", "\[Prime]",
          MultilineFunction->None], "[", 
         SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "2"]}], "-", 
      RowBox[{
       RowBox[{"v", "[", 
        SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], " ", 
       RowBox[{"vLim", "[", 
        SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
          SuperscriptBox[
           RowBox[{
            SuperscriptBox["v", "\[Prime]",
             MultilineFunction->None], "[", 
            SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "2"]}], "+", 
         RowBox[{
          RowBox[{"vLim", "[", 
           SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox["v", "\[Prime]",
              MultilineFunction->None], "[", 
             SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "+", 
            RowBox[{
             SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
             RowBox[{
              SuperscriptBox["v", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", 
              SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}]}], 
           ")"}]}]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"v", "[", 
         SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["vLim", "\[Prime]",
           MultilineFunction->None], "[", 
          SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "+", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
          RowBox[{
           SuperscriptBox["vLim", "\[Prime]\[Prime]",
            MultilineFunction->None], "[", 
           SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}]}], ")"}]}], 
      "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"v", "[", 
         SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              SuperscriptBox["v", "\[Prime]",
               MultilineFunction->None], "[", 
              SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], "-", 
            RowBox[{
             SuperscriptBox["vLim", "\[Prime]",
              MultilineFunction->None], "[", 
             SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], ")"}], " ", 
          
          RowBox[{
           SuperscriptBox["vLim", "\[Prime]",
            MultilineFunction->None], "[", 
           SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], "+", 
         RowBox[{
          RowBox[{"vLim", "[", 
           SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox["v", "\[Prime]",
              MultilineFunction->None], "[", 
             SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "+", 
            RowBox[{
             SuperscriptBox["vLim", "\[Prime]",
              MultilineFunction->None], "[", 
             SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "+", 
            RowBox[{
             SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                SuperscriptBox["v", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", 
                SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "+", 
               RowBox[{
                SuperscriptBox["vLim", "\[Prime]\[Prime]",
                 MultilineFunction->None], "[", 
                SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], 
              ")"}]}]}], ")"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"v", "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"v", "[", 
        SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "-", 
       RowBox[{"vLim", "[", 
        SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], ")"}], "2"]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{{3.502318834925703*^9, 3.502318869477769*^9}, 
   3.502318917879631*^9, 3.502319264638731*^9, 3.5023197588440742`*^9, 
   3.502449466785077*^9, 3.503944152108686*^9, 3.5047804169564943`*^9, 
   3.515057966364687*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InputForm", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"vGapPP", " ", "/.", " ", 
            RowBox[{
             SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", "\[Rule]", 
             "m"}]}], " ", "/.", " ", 
           RowBox[{
            RowBox[{
             SuperscriptBox["vLim", "\[Prime]\[Prime]",
              MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
            "vLimPPVecExcBot"}]}], " ", "/.", " ", 
          RowBox[{
           RowBox[{
            SuperscriptBox["vLim", "\[Prime]",
             MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
           "vLimPVecExcBot"}]}], "/.", " ", 
         RowBox[{
          RowBox[{"vLim", "[", "m", "]"}], " ", "\[Rule]", " ", 
          "vLimVecExcBot"}]}], "/.", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["u", "\[Prime]\[Prime]",
           MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
         RowBox[{"uPP", "[", "m", "]"}]}]}], " ", "/.", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["u", "\[Prime]",
          MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
        RowBox[{"uP", "[", "m", "]"}]}]}], " ", "/.", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["v", "\[Prime]\[Prime]",
         MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
       "vPPVecExcBot"}]}], " ", "/.", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["v", "\[Prime]",
        MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
      "vPVecExcBot"}]}], " ", "/.", " ", 
    RowBox[{
     RowBox[{"v", "[", "m", "]"}], " ", "\[Rule]", " ", "vVecExcBot"}]}], " ",
    "/.", " ", 
   RowBox[{"m", " ", "\[Rule]", " ", "mVecExcBot"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5023189227339973`*^9, 3.502318982182555*^9}, 
   3.5024449875852137`*^9, 3.502449473965385*^9}],

Cell["\<\
(mVecExcBot*(mVecExcBot*vLimVecExcBot^2*
    vPVecExcBot^2 - vLimVecExcBot*
    (2*mVecExcBot*vPVecExcBot^2 + 
     vLimVecExcBot*(mVecExcBot*vPPVecExcBot + 
       vPVecExcBot))*vVecExcBot + 
   (vLimVecExcBot*(vLimPVecExcBot + 
       mVecExcBot*(vLimPPVecExcBot + 
         vPPVecExcBot) + vPVecExcBot) + 
     mVecExcBot*vLimPVecExcBot*(-vLimPVecExcBot + 
       2*vPVecExcBot))*vVecExcBot^2 - 
   (mVecExcBot*vLimPPVecExcBot + vLimPVecExcBot)*
    vVecExcBot^3))/(vVecExcBot^2*
  (-vLimVecExcBot + vVecExcBot)^2)\
\>", "Output",
 CellChangeTimes->{3.50231894123139*^9, 3.5023189827840157`*^9, 
  3.502319269512705*^9, 3.502319773943796*^9, 3.502449476906714*^9, 
  3.503944153030582*^9, 3.504780418199025*^9, 3.515057967815495*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "\"\<Global`*\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "/", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"Exp", "[", 
          RowBox[{"vGap", "[", 
           RowBox[{"Log", "[", "m", "]"}], "]"}], "]"}]}], "   ", ")"}]}], 
      ")"}], 
     RowBox[{"\[ScriptV]Lim", "[", "m", "]"}]}], ",", "m"}], "]"}], "  ", "/.",
   " ", 
  RowBox[{
   RowBox[{
    SuperscriptBox["\[ScriptV]Lim", "\[Prime]",
     MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
   RowBox[{
   "\[ScriptV]PLim", "[", "m", "]"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.502462526108643*^9, 3.502462552333447*^9}, {
  3.502462582823119*^9, 3.502462585104995*^9}, {3.5024626212333927`*^9, 
  3.502462748364978*^9}, {3.502462973609047*^9, 3.502462982962584*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"\[ScriptV]PLim", "[", "m", "]"}], 
   RowBox[{"1", "-", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"vGap", "[", 
      RowBox[{"Log", "[", "m", "]"}], "]"}]]}]], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"vGap", "[", 
      RowBox[{"Log", "[", "m", "]"}], "]"}]], " ", 
    RowBox[{"\[ScriptV]Lim", "[", "m", "]"}], " ", 
    RowBox[{
     SuperscriptBox["vGap", "\[Prime]",
      MultilineFunction->None], "[", 
     RowBox[{"Log", "[", "m", "]"}], "]"}]}], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"vGap", "[", 
         RowBox[{"Log", "[", "m", "]"}], "]"}]]}], ")"}], "2"], " ", 
    "m"}]]}]], "Output",
 CellChangeTimes->{3.502462683523288*^9, 3.502462716298504*^9, 
  3.502462748925931*^9, 3.5024629750867777`*^9, 3.503944154424067*^9, 
  3.515057969301196*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", "endCDCPrivate", " ", "*)"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ClearAll", "[", "\"\<Global`*\>\"", "]"}], ";"}], "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
   "\[Chi]", " ", "is", " ", "discussed", " ", "in", " ", 
    "SolnMethTermFunc"}], " ", "*)"}], "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Relative", " ", "to", " ", "the", " ", "treatment", " ", "in", " ", 
     "that", " ", "document"}], ",", " ", 
    RowBox[{"two", " ", "modifications", " ", "are", " ", "incorporated", " ", 
     RowBox[{"here", ":", "\n", "\n", 
      RowBox[{
       RowBox[{"--", " ", "In"}], " ", "the", " ", "presence", " ", "of", " ",
        "liquidity", " ", "constraints"}]}]}], ",", " ", 
    RowBox[{
     RowBox[{
     "it", " ", "will", " ", "sometimes", " ", "be", " ", "the", " ", "case", 
      " ", "that", " ", 
      RowBox[{"\[ScriptC]", "[", "m", "]"}]}], "=", 
     RowBox[{"\[ScriptC]Lim", "[", "m", "]"}]}], ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
      "so", " ", "that", " ", "the", " ", "log", " ", "of", " ", "1"}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"\[ScriptC]", "[", "m", "]"}], "/", 
        RowBox[{"\[ScriptC]Lim", "[", "m", "]"}]}], " ", "would", " ", "be", 
       " ", "undefined"}]}], ";", " ", 
     RowBox[{
     "this", " ", "problem", " ", "is", " ", "solved", " ", "by", " ", 
      "adding", " ", "\[CurlyEpsilon]", " ", "to", " ", "the", " ", "formula",
       " ", 
      RowBox[{"(", 
       RowBox[{
       "with", " ", "a", " ", "corresponding", " ", "subtraction", " ", "in", 
        " ", "the", " ", "redefinition", " ", "of", " ", "\[ScriptC]"}], 
       ")"}]}], ";", "\n", "\n", 
     RowBox[{"--", " ", "Relatedly"}]}], ",", " ", 
    RowBox[{
     RowBox[{
     "if", " ", "there", " ", "is", " ", "a", " ", "liquidity", " ", 
      "constraint", " ", "so", " ", "that", " ", "\[Theta]Min"}], " ", ">", 
     " ", "0."}], ",", " ", 
    RowBox[{
    "it", " ", "is", " ", "better", " ", "to", " ", "approximate", " ", "the",
      " ", "result", " ", "of", " ", "dividing", " ", "\[ScriptC]", " ", "by",
      " ", "a", " ", "denominator", " ", "that", " ", "incorporates", " ", 
     "that", " ", 
     RowBox[{"\[Theta]Min", ".", "  ", 
      RowBox[{"(", 
       RowBox[{"Again", ",", " ", 
        RowBox[{"of", " ", "course"}], ",", " ", 
        RowBox[{
        "this", " ", "is", " ", "unwound", " ", "in", " ", "the", " ", 
         "definition", " ", "of", " ", "\[ScriptC]"}]}], ")"}], "."}]}]}], 
   "\n", "\n", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Chi]", " ", "=", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"\[CurlyEpsilon]", "+", "1", "-", 
      RowBox[{
       RowBox[{"\[ScriptC]", "[", 
        RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}], "/", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[ScriptC]T", "[", 
          RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}], "+", "\[Theta]Min"}], 
        ")"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{"d\[Chi]d\[Mu]", " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{"\[CurlyEpsilon]", "+", "1", "-", 
           RowBox[{
            RowBox[{"\[ScriptC]", "[", 
             RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}], "/", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"\[ScriptC]T", "[", 
               RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}], "+", 
              "\[Theta]Min"}], ")"}]}]}], "]"}], ",", "\[Mu]"}], "]"}], " ", "/.",
        " ", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", "\[Rule]", "m"}]}], 
      "  ", "/.", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[ScriptC]", "\[Prime]",
         MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
       "\[Kappa]"}]}], " ", "/.", " ", 
     RowBox[{"\[ScriptC]T", " ", "\[Rule]", " ", "\[ScriptC]Lim"}]}], " ", "/.",
     " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ScriptC]Lim", "\[Prime]",
       MultilineFunction->None], "[", "m", "]"}], " ", "\[Rule]", " ", 
     RowBox[{"\[Kappa]Lim", "[", "m", "]"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{"\[Chi]P", " ", "\[Equal]", " ", "d\[Chi]d\[Mu]"}], ",", 
     "\[Kappa]"}], "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"\[Chi]P", " ", "\[Rule]", " ", 
   RowBox[{"\[Chi]Prime", "[", 
    RowBox[{"Log", "[", "m", "]"}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.515164452084012*^9, 3.5151644738796177`*^9}, {
  3.515164505605433*^9, 3.515164515234914*^9}}],

Cell[BoxData[
 RowBox[{"\[Kappa]", "\[Rule]", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Theta]Min", "+", 
      RowBox[{"\[ScriptC]Lim", "[", "m", "]"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", "\[CurlyEpsilon]", "-", 
      FractionBox[
       RowBox[{"\[ScriptC]", "[", "m", "]"}], 
       RowBox[{"\[Theta]Min", "+", 
        RowBox[{"\[ScriptC]Lim", "[", "m", "]"}]}]]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"m", " ", 
        RowBox[{"\[ScriptC]", "[", "m", "]"}], " ", 
        RowBox[{"\[Kappa]Lim", "[", "m", "]"}]}], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"\[Theta]Min", "+", 
           RowBox[{"\[ScriptC]Lim", "[", "m", "]"}]}], ")"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", "\[CurlyEpsilon]", "-", 
          FractionBox[
           RowBox[{"\[ScriptC]", "[", "m", "]"}], 
           RowBox[{"\[Theta]Min", "+", 
            RowBox[{"\[ScriptC]Lim", "[", "m", "]"}]}]]}], ")"}]}]], "-", 
      RowBox[{"\[Chi]Prime", "[", 
       RowBox[{"Log", "[", "m", "]"}], "]"}]}], ")"}]}], "m"]}]], "Output",
 CellChangeTimes->{{3.503927803822977*^9, 3.5039278202436657`*^9}, {
   3.503927886824994*^9, 3.503927920938599*^9}, {3.503944155621166*^9, 
   3.503944179472857*^9}, 3.50394422283657*^9, 3.515057973687807*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", 
     RowBox[{
      RowBox[{"\[ScriptC]Now", "[", 
       RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}], "/", 
      RowBox[{"\[ScriptC]T", "[", 
       RowBox[{"Exp", "[", "\[Mu]", "]"}], "]"}]}]}], "]"}], ",", "\[Mu]"}], 
  "]"}]], "Input"],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
      RowBox[{
       SuperscriptBox["\[ScriptC]Now", "\[Prime]",
        MultilineFunction->None], "[", 
       SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], 
     RowBox[{"\[ScriptC]T", "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], " ", 
     RowBox[{"\[ScriptC]Now", "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], " ", 
     RowBox[{
      SuperscriptBox["\[ScriptC]T", "\[Prime]",
       MultilineFunction->None], "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]}], 
    SuperscriptBox[
     RowBox[{"\[ScriptC]T", "[", 
      SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], "2"]]}], 
  RowBox[{"1", "-", 
   FractionBox[
    RowBox[{"\[ScriptC]Now", "[", 
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}], 
    RowBox[{"\[ScriptC]T", "[", 
     SuperscriptBox["\[ExponentialE]", "\[Mu]"], "]"}]]}]]], "Output",
 CellChangeTimes->{3.5150579758063517`*^9}]
}, Open  ]]
},
WindowSize->{1436, 856},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (February 23, \
2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 496, 9, 87, "Input"],
Cell[CellGroupData[{
Cell[1078, 33, 1365, 38, 87, "Input"],
Cell[2446, 73, 334, 9, 53, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2817, 87, 1016, 27, 187, "Input"],
Cell[3836, 116, 258, 8, 68, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4131, 129, 157, 4, 40, "Input"],
Cell[4291, 135, 648, 22, 68, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4976, 162, 677, 19, 87, "Input"],
Cell[5656, 183, 209, 5, 79, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5902, 193, 274, 6, 40, "Input"],
Cell[6179, 201, 1311, 43, 73, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7527, 249, 1126, 30, 133, "Input"],
Cell[8656, 281, 389, 10, 177, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9082, 296, 482, 13, 87, "Input"],
Cell[9567, 311, 580, 17, 69, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10184, 333, 521, 14, 40, "Input"],
Cell[10708, 349, 179, 2, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10924, 356, 1209, 32, 87, "Input"],
Cell[12136, 390, 216, 5, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12389, 400, 226, 5, 40, "Input"],
Cell[12618, 407, 1137, 33, 73, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13792, 445, 1307, 34, 87, "Input"],
Cell[15102, 481, 441, 10, 118, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15580, 496, 628, 15, 64, "Input"],
Cell[16211, 513, 393, 9, 67, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16641, 527, 300, 8, 40, "Input"],
Cell[16944, 537, 188, 4, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17169, 546, 354, 7, 40, "Input"],
Cell[17526, 555, 1264, 33, 111, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18827, 593, 1851, 48, 87, "Input"],
Cell[20681, 643, 491, 9, 79, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21209, 657, 362, 7, 40, "Input"],
Cell[21574, 666, 5045, 130, 80, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26656, 801, 2029, 53, 110, "Input"],
Cell[28688, 856, 746, 17, 274, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29471, 878, 942, 26, 87, "Input"],
Cell[30416, 906, 966, 28, 77, "Output"]
}, Open  ]],
Cell[31397, 937, 73, 1, 40, "Input"],
Cell[CellGroupData[{
Cell[31495, 942, 4700, 121, 409, "Input"],
Cell[36198, 1065, 1366, 34, 98, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37601, 1104, 330, 10, 40, "Input"],
Cell[37934, 1116, 1151, 31, 111, "Output"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
